import * as React from 'react';
import { SandboxProps, SandboxContructor } from '@ice/sandbox';
import { AppHistory } from './appHistory';
import ModuleLoader from './util/umdLoader';
interface AppRouteState {
    cssLoading: boolean;
    showComponent: boolean;
}
declare type hashType = 'hashbang' | 'noslash' | 'slash';
interface Match<Params extends {
    [K in keyof Params]?: string;
} = {}> {
    params: Params;
    isExact: boolean;
    path: string;
    url: string;
}
interface Location<Query extends {
    [K in keyof Query]?: string;
} = {}> {
    pathname: string;
    query: Query;
    hash: string;
}
export interface AppRouteComponentProps<Params extends {
    [K in keyof Params]?: string;
} = {}> {
    match: Match<Params>;
    location: Location;
    history: AppHistory;
}
export interface PathData {
    value: string;
    exact?: boolean;
    strict?: boolean;
    sensitive?: boolean;
}
export interface AppConfig {
    sandbox?: boolean | SandboxProps | SandboxContructor;
    title?: string;
    hashType?: boolean | hashType;
    basename?: string;
    exact?: boolean;
    strict?: boolean;
    sensitive?: boolean;
    rootId?: string;
    path: string | (string | PathData)[];
    url?: string | string[];
    entry?: string;
    entryContent?: string;
    component?: React.ReactElement;
    render?: (props?: AppRouteComponentProps) => React.ReactElement;
    cache?: boolean;
    umd?: boolean;
    name?: string;
    customProps?: object;
}
export interface AppRouteProps extends AppConfig {
    onAppEnter?: (appConfig: AppConfig) => void;
    onAppLeave?: (appConfig: AppConfig) => void;
    triggerLoading?: (loading: boolean) => void;
    triggerError?: (err: string) => void;
    shouldAssetsRemove?: (assetUrl?: string, element?: HTMLElement | HTMLLinkElement | HTMLStyleElement | HTMLScriptElement) => boolean;
    componentProps?: AppRouteComponentProps;
    clearCacheRoot?: () => void;
    moduleLoader?: ModuleLoader;
}
export declare function converArray2String(list: string | (string | PathData)[]): string;
export default class AppRoute extends React.Component<AppRouteProps, AppRouteState> {
    state: {
        cssLoading: boolean;
        showComponent: boolean;
    };
    private myRefBase;
    private appSandbox;
    private unmounted;
    private prevAppConfig;
    private rootElement;
    static defaultProps: {
        exact: boolean;
        strict: boolean;
        sensitive: boolean;
        sandbox: boolean;
        rootId: string;
        shouldAssetsRemove: () => boolean;
    };
    componentDidMount(): void;
    shouldComponentUpdate(nextProps: any, nextState: any): boolean;
    componentDidUpdate(prevProps: any): void;
    componentWillUnmount(): void;
    /**
     * get cache key
     */
    getCacheKey: (appConfig?: AppConfig) => string;
    /**
     * Load assets and render sub-application
     */
    renderChild: () => void;
    loadNextApp: () => Promise<void>;
    reCreateElementInBase: (elementId: string) => HTMLElement;
    /**
     * Trigger onAppLeave in AppRouter and callAppLeave(registerAppLeave callback)
     * reset this.prevAppConfig
     */
    triggerPrevAppLeave: () => void;
    /**
     * Trigger onAppEnter in AppRouter
     * callAppEnter(registerAppEnter callback) will be triggered later
     * record current appConfig as this.prevAppConfig
     */
    triggerOnAppEnter: () => AppConfig;
    render(): JSX.Element;
}
export {};
