import * as React from 'react';
import { AppConfig } from './AppRoute';
declare type RouteType = 'pushState' | 'replaceState';
export interface AppRouterProps {
    onRouteChange?: (pathname: string, query: object, hash?: string, type?: RouteType | 'init' | 'popstate') => void;
    ErrorComponent?: any;
    LoadingComponent?: any;
    NotFoundComponent?: any;
    onAppEnter?: (appConfig: AppConfig) => void;
    onAppLeave?: (appConfig: AppConfig) => void;
    shouldAssetsRemove?: (assetUrl?: string, element?: HTMLElement | HTMLLinkElement | HTMLStyleElement | HTMLScriptElement) => boolean;
    basename?: string;
}
interface AppRouterState {
    url: string;
    showLoading: boolean;
}
export default class AppRouter extends React.Component<AppRouterProps, AppRouterState> {
    private unmounted;
    private err;
    static defaultProps: {
        onRouteChange: () => void;
        ErrorComponent: ({ err }: {
            err: any;
        }) => JSX.Element;
        NotFoundComponent: JSX.Element;
        shouldAssetsRemove: () => boolean;
        basename: string;
    };
    constructor(props: AppRouterProps);
    componentDidMount(): void;
    componentWillUnmount(): void;
    /**
     * Trigger NotFound
     */
    triggerNotFound: () => void;
    /**
     * Trigger Loading
     */
    triggerLoading: (newShowLoading: boolean) => void;
    /**
     * Trigger Error
     */
    triggerError: (err: string) => void;
    /**
     * Hijack window.history
     */
    hijackHistory: () => void;
    /**
     * Unhijack window.history
     */
    unHijackHistory: () => void;
    /**
     * Hijack eventListener
     */
    hijackEventListener: () => void;
    /**
     * Unhijack eventListener
     */
    unHijackEventListener: () => void;
    /**
     * Trigger statechange: pushState | replaceState
     */
    handleStateChange: (evt: PopStateEvent, url: string, routeType?: RouteType) => void;
    /**
     * Trigger popstate
     */
    handlePopState: (event: PopStateEvent) => void;
    /**
     * Trigger onRouteChange
     */
    handleRouteChange: (url: string, type: RouteType | 'init' | 'popstate') => void;
    clearCacheRoot: () => void;
    render(): JSX.Element;
}
export {};
