"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var urlParse = require("url-parse");
var appHistory_1 = require("./appHistory");
var renderComponent_1 = require("./util/renderComponent");
var matchPath_1 = require("./util/matchPath");
var handleAssets_1 = require("./util/handleAssets");
var constant_1 = require("./util/constant");
var cache_1 = require("./util/cache");
var capturedListeners_1 = require("./util/capturedListeners");
function addLeadingSlash(path) {
    return path.charAt(0) === '/' ? path : "/" + path;
}
var HashPathDecoders = {
    hashbang: function (path) { return (path.charAt(0) === '!' ? path.substr(1) : path); },
    noslash: addLeadingSlash,
    slash: addLeadingSlash,
};
function getHashPath(hash) {
    if (hash === void 0) { hash = '/'; }
    var hashIndex = hash.indexOf('#');
    var hashPath = hashIndex === -1 ? hash : hash.substr(hashIndex + 1);
    // remove hash query
    var searchIndex = hashPath.indexOf('?');
    return searchIndex === -1 ? hashPath : hashPath.substr(0, searchIndex);
}
var originalPush = window.history.pushState;
var originalReplace = window.history.replaceState;
var originalAddEventListener = window.addEventListener;
var originalRemoveEventListener = window.removeEventListener;
var AppRouter = /** @class */ (function (_super) {
    __extends(AppRouter, _super);
    function AppRouter(props) {
        var _this = _super.call(this, props) || this;
        _this.unmounted = false;
        _this.err = ''; // js assets load err
        /**
         * Trigger NotFound
         */
        _this.triggerNotFound = function () {
            // if AppRouter is unmounted, cancel all operations
            if (_this.unmounted)
                return;
            _this.setState({ url: constant_1.ICESTSRK_NOT_FOUND });
        };
        /**
         * Trigger Loading
         */
        _this.triggerLoading = function (newShowLoading) {
            // if AppRouter is unmounted, cancel all operations
            if (_this.unmounted)
                return;
            // if no LoadingComponent, showLoading will never be true
            if (newShowLoading && !_this.props.LoadingComponent)
                return;
            var showLoading = _this.state.showLoading;
            if (showLoading !== newShowLoading) {
                _this.setState({ showLoading: newShowLoading });
            }
        };
        /**
         * Trigger Error
         */
        _this.triggerError = function (err) {
            // if AppRouter is unmounted, cancel all operations
            if (_this.unmounted)
                return;
            _this.err = err;
            _this.setState({ url: constant_1.ICESTSRK_ERROR });
        };
        /**
         * Hijack window.history
         */
        _this.hijackHistory = function () {
            window.history.pushState = function (state, title, url) {
                var rest = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    rest[_i - 3] = arguments[_i];
                }
                originalPush.apply(window.history, __spreadArrays([state, title, url], rest));
                _this.handleStateChange(capturedListeners_1.createPopStateEvent(state, 'pushState'), url, 'pushState');
            };
            window.history.replaceState = function (state, title, url) {
                var rest = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    rest[_i - 3] = arguments[_i];
                }
                originalReplace.apply(window.history, __spreadArrays([state, title, url], rest));
                _this.handleStateChange(capturedListeners_1.createPopStateEvent(state, 'replaceState'), url, 'replaceState');
            };
            window.addEventListener('popstate', _this.handlePopState, false);
        };
        /**
         * Unhijack window.history
         */
        _this.unHijackHistory = function () {
            window.history.pushState = originalPush;
            window.history.replaceState = originalReplace;
            window.removeEventListener('popstate', _this.handlePopState, false);
        };
        /**
         * Hijack eventListener
         */
        _this.hijackEventListener = function () {
            window.addEventListener = function (eventName, fn) {
                var rest = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    rest[_i - 2] = arguments[_i];
                }
                if (typeof fn === 'function' &&
                    capturedListeners_1.routingEventsListeningTo.indexOf(eventName) >= 0 &&
                    !capturedListeners_1.isInCapturedEventListeners(eventName, fn)) {
                    capturedListeners_1.addCapturedEventListeners(eventName, fn);
                    return;
                }
                return originalAddEventListener.apply(window, __spreadArrays([eventName, fn], rest));
            };
            window.removeEventListener = function (eventName, listenerFn) {
                var rest = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    rest[_i - 2] = arguments[_i];
                }
                if (typeof listenerFn === 'function' && capturedListeners_1.routingEventsListeningTo.indexOf(eventName) >= 0) {
                    capturedListeners_1.removeCapturedEventListeners(eventName, listenerFn);
                    return;
                }
                return originalRemoveEventListener.apply(window, __spreadArrays([eventName, listenerFn], rest));
            };
        };
        /**
         * Unhijack eventListener
         */
        _this.unHijackEventListener = function () {
            window.addEventListener = originalAddEventListener;
            window.removeEventListener = originalRemoveEventListener;
        };
        /**
         * Trigger statechange: pushState | replaceState
         */
        _this.handleStateChange = function (evt, url, routeType) {
            // if AppRouter is unmounted, cancel all operations
            if (_this.unmounted)
                return;
            // setHistoryState before setState
            // setState is only async batched when it is called inside a React event handler, otherwise it is sync
            // make sure history state had beed recorded before render
            capturedListeners_1.setHistoryEvent(evt);
            _this.setState({ url: url });
            _this.handleRouteChange(url, routeType);
        };
        /**
         * Trigger popstate
         */
        _this.handlePopState = function (event) {
            // if AppRouter is unmounted, cancel all operations
            if (_this.unmounted)
                return;
            var url = location.href;
            capturedListeners_1.setHistoryEvent(event);
            _this.setState({ url: url });
            _this.handleRouteChange(url, 'popstate');
        };
        /**
         * Trigger onRouteChange
         */
        _this.handleRouteChange = function (url, type) {
            var _a = urlParse(url, true), pathname = _a.pathname, query = _a.query, hash = _a.hash;
            _this.props.onRouteChange(pathname, query, hash, type);
        };
        _this.clearCacheRoot = function () {
            // child unmout is called after parent unmount
            // if AppRouter is unmounted remove cache root
            if (_this.unmounted) {
                cache_1.setCache('root', null);
            }
        };
        _this.state = {
            url: location.href,
            showLoading: false,
        };
        handleAssets_1.recordAssets();
        return _this;
    }
    AppRouter.prototype.componentDidMount = function () {
        // render NotFoundComponent eventListener
        window.addEventListener('icestark:not-found', this.triggerNotFound);
        this.hijackHistory();
        this.hijackEventListener();
        this.handleRouteChange(location.href, 'init');
    };
    AppRouter.prototype.componentWillUnmount = function () {
        this.unmounted = true;
        var shouldAssetsRemove = this.props.shouldAssetsRemove;
        this.unHijackEventListener();
        this.unHijackHistory();
        window.removeEventListener('icestark:not-found', this.triggerNotFound);
        // empty all assets
        handleAssets_1.emptyAssets(shouldAssetsRemove, true);
    };
    AppRouter.prototype.render = function () {
        var _a = this.props, NotFoundComponent = _a.NotFoundComponent, ErrorComponent = _a.ErrorComponent, LoadingComponent = _a.LoadingComponent, onAppEnter = _a.onAppEnter, onAppLeave = _a.onAppLeave, shouldAssetsRemove = _a.shouldAssetsRemove, children = _a.children, appBasename = _a.basename;
        var _b = this.state, url = _b.url, showLoading = _b.showLoading;
        // directly render NotFoundComponent / ErrorComponent
        if (url === constant_1.ICESTSRK_NOT_FOUND) {
            return renderComponent_1.default(NotFoundComponent, {});
        }
        else if (url === constant_1.ICESTSRK_ERROR) {
            return renderComponent_1.default(ErrorComponent, { err: this.err });
        }
        var _c = urlParse(url, true), pathname = _c.pathname, query = _c.query, hash = _c.hash;
        var match = null;
        var element;
        React.Children.forEach(children, function (child) {
            if (match == null && React.isValidElement(child)) {
                element = child;
                var _a = child.props, path = _a.path, hashType = _a.hashType;
                var routerPath = appBasename
                    ? [].concat(path).map(function (pathStr) { return "" + addLeadingSlash(appBasename) + (pathStr.value || pathStr); })
                    : path;
                if (hashType) {
                    var decodePath = HashPathDecoders[hashType === true ? 'slash' : hashType];
                    var hashPath = decodePath(getHashPath(hash));
                    match = path ? matchPath_1.default(hashPath, __assign(__assign({}, child.props), { path: routerPath })) : null;
                }
                else {
                    match = path ? matchPath_1.default(pathname, __assign(__assign({}, child.props), { path: routerPath })) : null;
                }
            }
        });
        if (match) {
            var _d = element.props, path = _d.path, basename = _d.basename;
            var componentProps = {
                location: { pathname: pathname, query: query, hash: hash },
                match: match,
                history: appHistory_1.default,
            };
            // render AppRoute
            cache_1.setCache('basename', "" + appBasename + (basename || (Array.isArray(path) ? path[0].value || path[0] : path)));
            var extraProps = {
                onAppEnter: onAppEnter,
                onAppLeave: onAppLeave,
                triggerLoading: this.triggerLoading,
                triggerError: this.triggerError,
                clearCacheRoot: this.clearCacheRoot,
                shouldAssetsRemove: shouldAssetsRemove,
                componentProps: componentProps,
            };
            return (React.createElement("div", null,
                showLoading ? renderComponent_1.default(LoadingComponent, {}) : null,
                React.cloneElement(element, extraProps)));
        }
        return renderComponent_1.default(NotFoundComponent, {});
    };
    AppRouter.defaultProps = {
        onRouteChange: function () { },
        ErrorComponent: function (_a) {
            var err = _a.err;
            return React.createElement("div", null, err);
        },
        NotFoundComponent: React.createElement("div", null, "NotFound"),
        shouldAssetsRemove: function () { return true; },
        basename: '',
    };
    return AppRouter;
}(React.Component));
exports.default = AppRouter;
//# sourceMappingURL=AppRouter.js.map