"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.resetCapturedEventListeners = exports.removeCapturedEventListeners = exports.addCapturedEventListeners = exports.isInCapturedEventListeners = exports.setHistoryEvent = exports.callCapturedEventListeners = exports.createPopStateEvent = exports.find = exports.routingEventsListeningTo = exports.CapturedEventNameEnum = void 0;
//  Reference https://github.com/CanopyTax/single-spa/blob/master/src/navigation/navigation-events.js
var CapturedEventNameEnum;
(function (CapturedEventNameEnum) {
    CapturedEventNameEnum["POPSTATE"] = "popstate";
    CapturedEventNameEnum["HASHCHANGE"] = "hashchange";
})(CapturedEventNameEnum = exports.CapturedEventNameEnum || (exports.CapturedEventNameEnum = {}));
exports.routingEventsListeningTo = [
    CapturedEventNameEnum.HASHCHANGE,
    CapturedEventNameEnum.POPSTATE,
];
var capturedEventListeners = (_a = {},
    _a[CapturedEventNameEnum.POPSTATE] = [],
    _a[CapturedEventNameEnum.HASHCHANGE] = [],
    _a);
var historyEvent = null;
function find(list, element) {
    if (!Array.isArray(list)) {
        return false;
    }
    return list.filter(function (item) { return item === element; }).length > 0;
}
exports.find = find;
// inspired by https://github.com/single-spa/single-spa/blob/master/src/navigation/navigation-events.js#L107
function createPopStateEvent(state, originalMethodName) {
    // We need a popstate event even though the browser doesn't do one by default when you call replaceState, so that
    // all the applications can reroute.
    var evt;
    try {
        evt = new PopStateEvent('popstate', { state: state });
    }
    catch (err) {
        // IE 11 compatibility
        // https://docs.microsoft.com/en-us/openspecs/ie_standards/ms-html5e/bd560f47-b349-4d2c-baa8-f1560fb489dd
        evt = document.createEvent('PopStateEvent');
        evt.initPopStateEvent('popstate', false, false, state);
    }
    evt.icestark = true;
    evt.icestarkTrigger = originalMethodName;
    return evt;
}
exports.createPopStateEvent = createPopStateEvent;
function callCapturedEventListeners() {
    var _this = this;
    if (historyEvent) {
        Object.keys(capturedEventListeners).forEach(function (eventName) {
            var capturedListeners = capturedEventListeners[eventName];
            if (capturedListeners.length) {
                capturedListeners.forEach(function (listener) {
                    listener.call(_this, historyEvent);
                });
            }
        });
        historyEvent = null;
    }
}
exports.callCapturedEventListeners = callCapturedEventListeners;
function setHistoryEvent(evt) {
    historyEvent = evt;
}
exports.setHistoryEvent = setHistoryEvent;
function isInCapturedEventListeners(eventName, fn) {
    return find(capturedEventListeners[eventName], fn);
}
exports.isInCapturedEventListeners = isInCapturedEventListeners;
function addCapturedEventListeners(eventName, fn) {
    capturedEventListeners[eventName].push(fn);
}
exports.addCapturedEventListeners = addCapturedEventListeners;
function removeCapturedEventListeners(eventName, listenerFn) {
    capturedEventListeners[eventName] = capturedEventListeners[eventName].filter(function (fn) { return fn !== listenerFn; });
}
exports.removeCapturedEventListeners = removeCapturedEventListeners;
function resetCapturedEventListeners() {
    capturedEventListeners[CapturedEventNameEnum.POPSTATE] = [];
    capturedEventListeners[CapturedEventNameEnum.HASHCHANGE] = [];
}
exports.resetCapturedEventListeners = resetCapturedEventListeners;
//# sourceMappingURL=capturedListeners.js.map