import Sandbox from '@ice/sandbox';
export declare enum AssetTypeEnum {
    INLINE = "inline",
    EXTERNAL = "external"
}
export declare enum AssetCommentEnum {
    REPLACED = "replaced",
    PROCESSED = "processed"
}
export interface Asset {
    type: AssetTypeEnum;
    content: string;
}
export interface ProcessedContent {
    html: string;
    assets: Assets;
}
export interface Assets {
    jsList: Asset[];
    cssList: Asset[];
}
export interface ParsedConfig {
    origin: string;
    pathname: string;
}
export interface Fetch {
    (input: RequestInfo, init?: RequestInit): Promise<Response>;
}
/**
 * Create link/style element and append to root
 */
export declare function appendCSS(root: HTMLElement | ShadowRoot, asset: string | Asset, id: string): Promise<string>;
/**
 * Create script element (without inline) and append to root
 */
export declare function appendExternalScript(root: HTMLElement | ShadowRoot | DocumentFragment, asset: string | Asset, id: string): Promise<string>;
export declare function getUrlAssets(urls: string[]): {
    jsList: any[];
    cssList: any[];
};
export declare function fetchScripts(jsList: Asset[], fetch?: Fetch): Promise<any[]>;
export declare function appendAssets(assets: Assets, cacheKey: string, umd: boolean, sandbox?: Sandbox): Promise<void>;
export declare function parseUrl(entry: string): ParsedConfig;
export declare function startWith(url: string, prefix: string): boolean;
export declare function getUrl(entry: string, relativePath: string): string;
/**
 * If script/link processed by @ice/stark, add comment for it
 */
export declare function getComment(tag: string, from: string, type: AssetCommentEnum): string;
/**
 * html -> { html: processedHtml, assets: processedAssets }
 */
export declare function processHtml(html: string, entry?: string): ProcessedContent;
export declare function getEntryAssets({ root, entry, entryContent, assetsCacheKey, href, fetch, }: {
    root: HTMLElement | ShadowRoot;
    entry?: string;
    entryContent?: string;
    assetsCacheKey: string;
    href?: string;
    fetch?: Fetch;
    assertsCached?: boolean;
}): Promise<any>;
export declare function getAssetsNode(): (HTMLStyleElement | HTMLScriptElement)[];
/**
 * Record static assets
 */
export declare function recordAssets(): void;
/**
 * If `PREFIX` is setted in `DYNAMIC` type, remain it
 */
export declare function setStaticAttribute(tag: HTMLStyleElement | HTMLScriptElement): void;
/**
 * Empty useless assets
 */
export declare function emptyAssets(shouldRemove: (assetUrl: string, element?: HTMLElement | HTMLLinkElement | HTMLStyleElement | HTMLScriptElement) => boolean, cacheKey: string | boolean): void;
export declare function checkCacheKey(node: HTMLElement | HTMLLinkElement | HTMLStyleElement | HTMLScriptElement, cacheKey: string | boolean): boolean;
/**
 * cache all assets loaded by current sub-application
 */
export declare function cacheAssets(cacheKey: string): void;
