"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGobalWindow = void 0;
var global_1 = require("./global");
var handleAssets_1 = require("./handleAssets");
;
;
var ModuleLoader = /** @class */ (function () {
    function ModuleLoader() {
        this.importTask = {};
    }
    ModuleLoader.prototype.load = function (starkModule) {
        var jsList = starkModule.jsList, cacheKey = starkModule.cacheKey;
        if (this.importTask[cacheKey]) {
            // return promise if current module is pending or resolved
            return this.importTask[cacheKey];
        }
        var task = handleAssets_1.fetchScripts(jsList);
        this.importTask[cacheKey] = task;
        return task;
    };
    ModuleLoader.prototype.clearTask = function () {
        this.importTask = {};
    };
    ModuleLoader.prototype.execModule = function (starkModule, sandbox) {
        return this.load(starkModule).then(function (sources) {
            var globalWindow = getGobalWindow(sandbox);
            var cacheKey = starkModule.cacheKey;
            var libraryExport = '';
            // excute script in order
            try {
                sources.forEach(function (source, index) {
                    var lastScript = index === sources.length - 1;
                    if (lastScript) {
                        global_1.noteGlobalProps(globalWindow);
                    }
                    // check sandbox
                    if (sandbox === null || sandbox === void 0 ? void 0 : sandbox.execScriptInSandbox) {
                        sandbox.execScriptInSandbox(source);
                    }
                    else {
                        // https://developer.mozilla.org/zh-CN/docs/Web/JavaScript/Reference/Global_Objects/eval
                        // eslint-disable-next-line no-eval
                        (0, eval)(source);
                    }
                    if (lastScript) {
                        libraryExport = global_1.getGlobalProp(globalWindow);
                    }
                });
            }
            catch (err) {
                console.error(err);
            }
            var moduleInfo = libraryExport ? globalWindow[libraryExport] : (globalWindow[cacheKey] || {});
            // remove moduleInfo from globalWindow in case of excute multi module in globalWindow
            if (globalWindow[libraryExport]) {
                delete globalWindow[libraryExport];
            }
            return moduleInfo;
        });
    };
    return ModuleLoader;
}());
exports.default = ModuleLoader;
;
/**
 * Get globalwindow
 *
 * @export
 * @param {Sandbox} [sandbox]
 * @returns
 */
function getGobalWindow(sandbox) {
    if (sandbox === null || sandbox === void 0 ? void 0 : sandbox.getSandbox) {
        sandbox.createProxySandbox();
        return sandbox.getSandbox();
    }
    // FIXME: If run in Node environment
    return window;
}
exports.getGobalWindow = getGobalWindow;
//# sourceMappingURL=umdLoader.js.map